@@ -12,6 +12,26 @@ class ColumnAdder(BaseEstimator, TransformerMixin):
 
         :param column_list: list of columns to be added together
         :param new_column_name:str, name of new column created from adding the columns in column list together
+
+        Usage
+        df = pd.DataFrame({'A':[1,2,3,4,5],
+                   'B':[2,4,6,8,10],
+                   'C':['dog','cat', 'sheep','dog','cat'],
+                   'D':['male','male','male','female','female'],
+                   'E':[True,True,False,True,True]})
+
+        >>> column_adder = ColumnAdder(column_list=['A','B'],new_column_name='Z')
+        >>> added_df = column_adder.transform(X=df)
+        >>> print(added_df)
+
+           A   B      C       D      E   Z
+        0  1   2    dog    male   True   3
+        1  2   4    cat    male   True   6
+        2  3   6  sheep    male  False   9
+        3  4   8    dog  female   True  12
+        4  5  10    cat  female   True  15
+
+
         """
         self.column_list = column_list
         self.new_column_name = new_column_name
@@ -25,6 +45,7 @@ class ColumnAdder(BaseEstimator, TransformerMixin):
         if check_dataframe(X) \
                 and check_columns(X, self.column_list) \
                 and check_numeric(df=X, column_list=self.column_list):
+            X = X.copy()
             X[self.new_column_name] = X.loc[:, self.column_list].sum(axis=1)
 
         return X
@@ -37,6 +58,19 @@ class ColumnMultiplier(BaseEstimator,TransformerMixin):
 
         :param column_list:
         :param new_column_name:
+
+        Usage
+        >>> df3 = pd.DataFrame({'A':[1,1,2,1,1],'B':[2,2,1,2,0],})
+        >>> multiplied_df = ColumnMultiplier(column_list=['A','B'], new_column_name='Z').transform(X=df3)
+        >>> print(multiplied_df)
+           A  B  Z
+        0  1  2  2
+        1  1  2  2
+        2  2  1  2
+        3  1  2  2
+        4  1  0  0
+
+
         """
         self.column_list = column_list
         self.new_column_name = new_column_name
@@ -49,6 +83,7 @@ class ColumnMultiplier(BaseEstimator,TransformerMixin):
     def transform(self,X, y=None):
         if check_dataframe(X) and \
                 check_columns(X, self.column_list) and check_numeric(df=X, column_list=self.column_list):
+            X = X.copy()
             X[self.new_column_name] = X.loc[:, self.column_list].prod(axis=1,
                                                                       numeric_only=True,
                                                                       skipna=True)
@@ -59,21 +94,36 @@ class ColumnSubtraction(BaseEstimator, TransformerMixin):
     def __init__(self, left, right, new_column_name):
         """
 
-        :param left:
-        :param right:
-        :param new_column_name:
+        :param left: name of column on the left side of the minus side
+        :param right: name of column on the right side of the minus side
+        :param new_column_name:str, name of new column
+
+        Usage
+        >>> df3 = pd.DataFrame({'A':[1,1,2,1,1],'B':[2,2,1,2,0],})
+        >>> subtracted_df = ColumnSubtraction(left='A', right='B', new_column_name='C').transform(df3)
+        >>> print(subtracted_df)
+           A  B  C
+        0  1  2 -1
+        1  1  2 -1
+        2  2  1  1
+        3  1  2 -1
+        4  1  0  1
+
         """
         self.left = left
         self.right = right
         self.new_column_name = new_column_name
+        assert isinstance(left,str), "left must be a string with name of column"
+        assert isinstance(right, str), "right must be a string with name of column"
+        assert isinstance(new_column_name, str), "new_column_name must be a string"
 
     def fit(self, X, y=None):
         return self
 
     def transform(self, X, y=None):
         if check_dataframe(X) and \
-                check_columns(X, [self.left,self.right]) and \
                 check_numeric(df=X, column_list=[self.left,self.right]):
+            X = X.copy()
             X[self.new_column_name] = X.loc[:, self.left] - X.loc[:, self.right]
         return X
 
@@ -98,6 +148,7 @@ class ExpandingTransformer(BaseEstimator, TransformerMixin):
 
     def transform(self, X, y=None):
         if check_columns(column_list=self.column_list) and check_dataframe(X):
+            X = X.copy()
             for col in self.column_list:
                 X[col] = X[col].expanding(self.min_periods, axis=0).agg(self.aggregation_function)
 
@@ -106,14 +157,51 @@ class ExpandingTransformer(BaseEstimator, TransformerMixin):
 
 class GroupbyTransformer(BaseEstimator, TransformerMixin):
     def __init__(self, column_list, aggregation_method='mean'):
+        """
+
+        :param column_list:
+        :param aggregation_method:
+
+        Usage
+        >>> df4 = pd.DataFrame({'Age':[1,2,3,4,5],
+                   'Gender':['Male','Female','Unknown','Male','Female'],
+                    'City':['austin', 'austin', 'lagos','abuja', 'ibadan']
+                   })
+        >>> grouped_df = GroupbyTransformer(column_list=['Gender', 'City'], aggregation_method='mean').transform(df4)
+
+        >>> print(grouped_df)
+
+                        Age
+        Gender  City
+        Female  austin    2
+                ibadan    5
+        Male    abuja     4
+                austin    1
+        Unknown lagos     3
+
+        >>> grouped_df = GroupbyTransformer(column_list=['Gender'], aggregation_method='sum').transform(df4)
+
+        >>> print(grouped_df)
+
+                 Age
+        Gender
+        Female     7
+        Male       5
+        Unknown    3
+
+
+        """
         self.column_list = column_list
         self.aggregation_method = aggregation_method
+        assert isinstance(column_list, list)
+        assert aggregation_method in ['mean', 'std', 'sum']
 
     def fit(self, X, y=None):
         return self
 
     def transform(self, X, y=None):
         if check_dataframe(df=X) and check_columns(df=X, column_list=self.column_list):
+            X = X.copy()
             X = X.groupby(self.column_list).agg(self.aggregation_method)
 
             return X
