@@ -9,13 +9,16 @@ from .plot import (create_pie,
 from .utils import (columns_with_distinct_values,
                     extract_col_types,
                     figures_to_html)
+import pandas as pd
 
 
 class AutoPlot(object):
     def __init__(self,
                  df,
-                 nbins=None,
-                 marginals=None,
+                 nbins=30,
+                 scatter_maximum_color_groups=5,
+                 marginal_x=None,
+                 marginal_y=None,
                  cumulative=False,
                  filename='autoplot',
                  box_points='outliers',
@@ -24,7 +27,7 @@ class AutoPlot(object):
                  log_x=False,
                  log_y=False,
                  size=None,
-                 histfunc=None,
+                 histfunc='count',
                  histnorm=None,
                  include_scatter=False,
                  include_box=True,
@@ -44,7 +47,9 @@ class AutoPlot(object):
                  maximum_number_boxplots=5,
                  maximum_number_violinplots=5,
                  pie_chart_hole=False,
-                 create_html=True
+                 create_html=True,
+                 scatter_orientation='v',
+                 opacity = 1.0
                  ):
         """
         Entry point for automated data visualization
@@ -82,7 +87,11 @@ class AutoPlot(object):
         """
         self.df = df
         self.nbins = nbins
-        self.marginals = marginals
+        self.scatter_maximum_color_groups = scatter_maximum_color_groups
+        self.scatter_orientation = scatter_orientation
+        self.opacity=opacity
+        self.marginal_x = marginal_x
+        self.marginal_y = marginal_y
         self.cumulative = cumulative
         self.filename = filename
         self.points = box_points
@@ -115,6 +124,53 @@ class AutoPlot(object):
         self.pie_chart_hole = pie_chart_hole
         self.pie_sectors = columns_with_distinct_values(df=self.df, categorical_only=True,
                                                         maximum_number_distinct_values=self.maximum_number_sectors)
+        assert isinstance(df, pd.DataFrame), "df must be a pandas dataframe"
+        assert isinstance(nbins, int), "nbins must be an integer"
+        assert isinstance(scatter_maximum_color_groups, int)
+        assert marginal_x in ['rug', 'box', 'violin', 'histogram', None]
+        assert marginal_y in ['rug', 'box', 'violin', 'histogram', None]
+        assert isinstance(scatter_orientation, str)
+        assert scatter_orientation in ['v', 'h']
+        assert isinstance(cumulative, bool), "cumulative must be a boolean"
+        assert '.' not in filename, "filename doesn't need an extension"
+        assert isinstance(filename, str), "filename must be a string with a dot or an extension"
+        assert box_points in ['all', 'outliers', 'suspectedoutliers', False], "accepted values for points 'all', " \
+                                                                          "'outliers', " \
+                                                                          "'suspectedoutliers'"
+        assert isinstance(boxmode, str), "boxmode must be a string"
+        assert boxmode in ['group', 'overlay'], "boxmode must be either group or overlay"
+        assert isinstance(box_notched, bool), "notched must be a boolean"
+        assert isinstance(log_y, bool),"log_x must be a boolean"
+        assert isinstance(log_x, bool),"log_y must be a boolean"
+        assert isinstance(size, str)
+        assert histnorm in ['percent', 'probability', 'density', 'probability density', None]
+        assert histfunc in ['count', 'sum', 'avg', 'min',
+                            'max'], "histfunc must be one of 'count', 'sum', 'avg','min','max' "
+        assert violinmode in ['group', 'overlay'], "violinmode must be either group or overlay"
+        assert isinstance(violin_box, bool)
+        assert violin_points in ['all', 'outliers', 'suspectedoutliers', False]
+        assert treemap_path_limit > 1
+        assert sunburst_path_limit > 1
+        assert correlation_method in ['pearson', 'kendall', 'spearman']
+        assert maximum_number_sectors > 1
+        assert isinstance(maximum_number_sectors,int)
+        assert isinstance(maximum_number_boxplots,int)
+        assert isinstance(maximum_number_violinplots,int)
+        assert isinstance(pie_chart_hole, bool)
+        assert isinstance(opacity, float)
+        assert opacity <= 1.0
+        assert isinstance(include_pie, bool)
+        assert isinstance(include_treemap,bool)
+        assert isinstance(include_box, bool)
+        assert isinstance(include_correlation, bool)
+        assert isinstance(include_histogram, bool)
+        assert isinstance(include_scatter,bool)
+        assert isinstance(include_sunburst,bool)
+        assert isinstance(include_violin, bool)
+        assert isinstance(create_html,bool)
+        # assert marginal_x in ['rug', 'box', 'violin', 'histogram', None]
+        # assert marginal_y in ['rug', 'box', 'violin', 'histogram', None]
+        # assert isinstance(cumulative, bool), "cumulative must be a boolean"
 
     def chart_type(self):
         column_types = self.column_types
@@ -172,43 +228,36 @@ class AutoPlot(object):
 
                     figure_list.extend(pie_list)
                 if chart == 'scatter' and self.include_scatter:
-                    if len(categorical) == 0:
-                        scatter_plot_list = create_scatter(df=self.df,
-                                                           col_types= self.column_types,
-                                                             basic=True,
-                                                             marginal_x=self.marginals,
-                                                             marginal_y=self.marginals,
-                                                             log_x=self.log_x,
-                                                             log_y=self.log_y,
-                                                             create_html=False)
+                    # if len(categorical) == 0:
 
-                    else:
-                        scatter_plot_list = create_scatter(df=self.df,
-                                                           col_types= self.column_types,
-                                                             basic=False,
-                                                             marginal_x=self.marginals,
-                                                             marginal_y=self.marginals,
-                                                             log_x=self.log_x,
-                                                             log_y=self.log_y,
-                                                             create_html=False)
+                    scatter_plot_list = create_scatter(df=self.df,
+                                                       col_types= self.column_types,
+                                                       marginal_y=self.marginal_x,
+                                                       marginal_x=self.marginal_y,
+                                                       orientation=self.scatter_orientation,
+                                                       opacity=self.opacity,
+                                                       maximum_color_groups=self.scatter_maximum_color_groups,
+                                                       log_x=self.log_x,
+                                                       log_y=self.log_y,
+                                                       create_html=False)
 
                     figure_list.extend(scatter_plot_list)
 
                 if chart == 'correlation_plot' and self.include_correlation:
                     correlation_plot_list = create_correlation_plot(df=self.df,
-                                                                      correlation_method=self.correlation_method)
+                                                                    correlation_method=self.correlation_method)
                     figure_list.extend(correlation_plot_list)
 
                 if chart == 'histogram' and self.include_histogram:
                     histogram_list = create_histogram(df=self.df,
-                                                        numeric_cols=numeric_cols,
-                                                        nbins=self.nbins,
-                                                        marginal=self.marginals,
-                                                        cumulative=self.cumulative,
-                                                        histfunc=self.histfunc,
-                                                        histnorm=self.histnorm,
-                                                        filename='histogram',
-                                                        create_html=False)
+                                                      numeric_cols=numeric_cols,
+                                                      nbins=self.nbins,
+                                                      marginal=self.marginal_x,
+                                                      cumulative=self.cumulative,
+                                                      histfunc=self.histfunc,
+                                                      histnorm=self.histnorm,
+                                                      filename='histogram',
+                                                      create_html=False)
 
                     figure_list.extend(histogram_list)
 
@@ -227,20 +276,19 @@ class AutoPlot(object):
 
                 if chart == 'treemaps' and self.include_treemap:
                     treemap_list = create_treemap(df=self.df,
-                                                    col_types=self.column_types,
-                                                    create_html=False,
-                                                    filename='treemap',
-                                                    limit=self.treemap_path_limit
-                                                    )
+                                                  col_types=self.column_types,
+                                                  create_html=False,
+                                                  filename='treemap',
+                                                  limit=self.treemap_path_limit)
                     figure_list.extend(treemap_list)
 
                 if chart == 'sunburst' and self.include_sunburst:
                     sunburst_list = create_sunburst(df=self.df,
-                                                      col_types=self.column_types,
-                                                      create_html=False,
-                                                      filename='sunburst',
-                                                      limit=self.sunburst_path_limit
-                                                      )
+                                                    col_types=self.column_types,
+                                                    create_html=False,
+                                                    filename='sunburst',
+                                                    limit=self.sunburst_path_limit)
+
                     figure_list.extend(sunburst_list)
 
                 if chart == 'violinplot' and self.include_violin:
